#include <stdio.h>
#include "mboot.h"
#include "vid.h"
#include "term.h"
#include "asmops.h"
#include "segm.h"
#include "intr.h"
#include "mem.h"
#include "vm.h"

static void do_nothing();

void kmain(struct mboot_info *mbinf)
{
	clear_scr();

	/* pointless verbal diarrhea */
	if(mbinf->flags & MB_LDRNAME) {
		printf("loaded by: %s\n", mbinf->boot_loader_name);
	}
	if(mbinf->flags & MB_CMDLINE) {
		printf("kernel command line: %s\n", mbinf->cmdline);
	}

	puts("kernel starting up");

	init_segm();
	init_intr();

	/* silence the blasted timer interrupt */
	interrupt(32, do_nothing);

	/* initialize the physical memory manager */
	init_mem(mbinf);
	/* initialize paging and the virtual memory manager */
	init_vm();

	/* kernel initialization complete, enable interrupts */
	enable_intr();

	dbg_print_vm(MEM_USER);
	dbg_print_vm(MEM_KERNEL);
}

static void do_nothing()
{
}
